#include <math.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "nrutil.c"


#define SWAP(a,b) tempr=(a);(a)=(b);(b)=tempr

void four1(float data[], unsigned long nn, int isign)
/*
Replaces data[1..2*nn] by its discrete Fourier transform, if isign is input as 1; or replaces
data[1..2*nn] by nn times its inverse discrete Fourier transform, if isign is input as -1.
data is a complex array of length nn or, equivalently, a real array of length 2*nn. nn MUST
be an integer power of 2 (this is not checked for!).
*/
{
	unsigned long n,mmax,m,j,istep,i;
	double wtemp,wr,wpr,wpi,wi,theta;	//Double precision for the trigonometric recurrences. 
	float tempr,tempi;

	n=nn << 1;
	j=1;
	for (i=1;i<n;i+=2) {				//This is the bit-reversal section of the routine. 
		if (j > i) {
			SWAP(data[j],data[i]);		//Exchange the two complex numbers.
			SWAP(data[j+1],data[i+1]);
		}
		m=nn;
		while (m >= 2 && j > m) {
			j -= m;
			m >>= 1;
		}
		j += m;
	}

	//Here begins the Danielson-Lanczos section of the routine.

	mmax=2;
	while (n > mmax) {					//Outer loop executed log2 nn times.
		istep=mmax << 1;
		theta=isign*(6.28318530717959/mmax);	//Initialize the trigonometric recurrence.
		wtemp=sin(0.5*theta);
		wpr = -2.0*wtemp*wtemp;
		wpi=sin(theta);
		wr=1.0;
		wi=0.0;
		for (m=1;m<mmax;m+=2) {			//Here are the two nested inner loops.
			for (i=m;i<=n;i+=istep) {
				j=i+mmax;				//This is the Danielson-Lanczos formula:
				tempr=wr*data[j]-wi*data[j+1];
				tempi=wr*data[j+1]+wi*data[j];
				data[j]=data[i]-tempr;
				data[j+1]=data[i+1]-tempi;
				data[i] += tempr;
				data[i+1] += tempi;
			}
			wr=(wtemp=wr)*wpr-wi*wpi+wr; //Trigonometric recurrence.
			wi=wi*wpr+wtemp*wpi+wi;
		}
		mmax=istep;
	}
}


void main(int argc, char *argv[])
{
	int i, d, n2;
	FILE *fp;
	char *fn;
	unsigned long n;
	float *data, T;

	for (i=1; i<argc; i++ ){
		
		switch(i){
	    	case 1: 
				fn = argv[1]; 
				break;
			case 2:
				n = (unsigned long)atol(argv[2]); 
				n2 = (int)(2*n); 
				break;
			case 3:
				d = atoi(argv[3]); 
				break;
			case 4:
				T = atof(argv[4]); 
				break;
		} 
	}

	if((fp=fopen(fn,"r"))==NULL)
	{	
		printf("\n Error in opening the file %s for reading!", fn);
		exit(1);
	}

	data = vector(1,n2);

	for(i=1; i<=n2; i++) 
	{	
		fscanf(fp, "%f", &(data[i]) );
	}
	
	fclose(fp);

	four1(data, n, d);
	
	for(i=n+1; i<=n2; i=i+2)              /* negativ freq part */
	{	
		printf("%g \t %g \t %g \n", ((i-1 - n2)/2)/T, data[i], data[i+1] );
	}

	for(i=1; i<=n; i=i+2)				/* positive freq part */
	{	
		printf("%g \t %g \t %g \n", ((i-1)/2)/T, data[i], data[i+1] );
	}

	free_vector(data,1,n2);
}